#ifndef _XC3S_PowerReset_H
#define _XC3S_PowerReset_H

#include "Prefix.h"
#include "EARTH_PT.h"
#include "XC3S_Register.h"
#include "PCI_Device.h"

namespace EARTH {
namespace XC3S {
	class Register;

	class PowerReset {
	public:
		PowerReset(PCI::Device *device, XC3S::Register *register_) : mDevice(device), mRegister(register_)
		{
			_ASSERTE(mDevice);
			_ASSERTE(mRegister);

			mLnbPower = mLnbPowerWhenClose = PT::Device::LNB_POWER_OFF;
			mTunerPowerReset = PT::Device::TUNER_POWER_OFF;
		}

		~PowerReset()
		{
		}

		status SetLnbPower(PT::Device::LnbPower lnbPower)
		{
			status status;

			uint data;
			status = ConvertData(lnbPower, mTunerPowerReset, &data);
			if (status) return status;

			mRegister->Write4(A_WRITE, data);
			mLnbPower = lnbPower;

			return status;
		}

		status GetLnbPower(PT::Device::LnbPower *lnbPower) const
		{
			if (lnbPower == NULL) return PT::STATUS_INVALID_PARAM_ERROR;

			*lnbPower = mLnbPower;

			return PT::STATUS_OK;
		}

		status SetLnbPowerWhenClose(PT::Device::LnbPower lnbPower)
		{
			status status;

			uint data;
			status = ConvertData(lnbPower, PT::Device::TUNER_POWER_OFF, &data);
			if (status) return status;

			status = mDevice->SetReset(A_WRITE, data);
			if (status) return status;

			mLnbPowerWhenClose = lnbPower;

			return status;
		}

		status GetLnbPowerWhenClose(PT::Device::LnbPower *lnbPower) const
		{
			if (lnbPower == NULL) return PT::STATUS_INVALID_PARAM_ERROR;

			*lnbPower = mLnbPowerWhenClose;

			return PT::STATUS_OK;
		}

		status SetTunerPowerReset(PT::Device::TunerPowerReset tunerPowerReset)
		{
			status status;
			
			uint data;
			status = ConvertData(mLnbPower, tunerPowerReset, &data);
			if (status) return status;

			mRegister->Write4(A_WRITE, data);
			mTunerPowerReset = tunerPowerReset;

			return status;
		}

		status GetTunerPowerReset(PT::Device::TunerPowerReset *tunerPowerReset) const
		{
			if (tunerPowerReset == NULL) return PT::STATUS_INVALID_PARAM_ERROR;

			*tunerPowerReset = mTunerPowerReset;

			return PT::STATUS_OK;
		}

	protected:
		enum {
			A_WRITE = 0x01
		};

		enum {
			BIT_TUNER =  0,
			BIT_LNB_UP,
			BIT_LNB_DOWN,
			BIT_RESET
		};

		PCI::Device *const mDevice;
		XC3S::Register *const mRegister;
		PT::Device::LnbPower mLnbPower, mLnbPowerWhenClose;
		PT::Device::TunerPowerReset mTunerPowerReset;

		status ConvertData(PT::Device::LnbPower lnbPower, PT::Device::TunerPowerReset tunerPowerReset, uint *data_)
		{
			uint data = 0;

			switch (lnbPower) {
				case PT::Device::LNB_POWER_OFF: data |= BIT_1(0) << BIT_LNB_UP | BIT_1(0) << BIT_LNB_DOWN; break;
				case PT::Device::LNB_POWER_15V: data |= BIT_1(1) << BIT_LNB_UP | BIT_1(1) << BIT_LNB_DOWN; break;
				case PT::Device::LNB_POWER_11V: data |= BIT_1(0) << BIT_LNB_UP | BIT_1(1) << BIT_LNB_DOWN; break;
				default                       : return PT::STATUS_GENERAL_ERROR;
			}

			switch (tunerPowerReset) {
				case PT::Device::TUNER_POWER_OFF             : data |= BIT_1(0) << BIT_TUNER | BIT_1(0) << BIT_RESET; break;
				case PT::Device::TUNER_POWER_ON_RESET_ENABLE : data |= BIT_1(1) << BIT_TUNER | BIT_1(0) << BIT_RESET; break;
				case PT::Device::TUNER_POWER_ON_RESET_DISABLE: data |= BIT_1(1) << BIT_TUNER | BIT_1(1) << BIT_RESET; break;
				default                                      : return PT::STATUS_GENERAL_ERROR;
			}

			_ASSERTE(data_);
			*data_ = data;

			return PT::STATUS_OK;
		}
	};
}
}

#endif
